#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform float brightness;

uniform vec3 camera_world;

uniform struct Material {
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	float shininess;
} mat;

//uniform vec3 materialColor;
//uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
//uniform float specularAlpha;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, vec3 diffuseF, vec3 specularC, vec3 specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return ((diffuseF + diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att) * brightness;  
}



void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	//color = vec4(materialColor * materialCoefficients.x, 1); // ambient
	
	color = vec4(mat.ambient,1);

	// add directional light contribution
	//color.rgb += phong(n, -dirL.direction, v, dirL.color * materialColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
	color.rgb += phong(n, -dirL.direction, v, dirL.color * mat.ambient, mat.diffuse, dirL.color, mat.specular, mat.shininess, false, vec3(0));
				
	// add point light contribution
	//color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * materialColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * mat.ambient, mat.diffuse, pointL.color, mat.specular, mat.shininess, true, pointL.attenuation);

}