#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 boneIDs;
layout(location = 4) in vec4 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

const int MAX_BONES = 100;
const int MAX_BONE_INFLUENCE = 4;
uniform mat4 finalBoneMatrices[MAX_BONES];

void main() {
	vec4 totalLocalPos = vec4(0.0);
	vec4 totalNormal = vec4(0.0);
	
	for(int i=0;i<MAX_BONE_INFLUENCE;i++){
		mat4 boneTransform = finalBoneMatrices[boneIDs[i]];
		vec4 posePosition = boneTransform * vec4(position, 1.0);
		totalLocalPos += posePosition * weights[i];
		
		vec4 worldNormal = boneTransform * vec4(normal, 0.0);
		totalNormal += worldNormal * weights[i];
	}

	vert.normal_world = totalNormal.xyz;
	vert.uv = uv;
	vert.position_world = (modelMatrix * totalNormal).xyz;
	gl_Position = viewProjMatrix * modelMatrix * totalLocalPos;
}